#include <iostream>
#include <string>
#include <locale>
#include <exception>
#include <chrono>
#include <cstdint>
#include <stdexcept>

#include "mxnet2license.h"

using namespace std::chrono_literals;


namespace sampleApp{

    constexpr int32_t   USERCODE = 1234;
    constexpr int16_t   APPSLOT = 7;
    constexpr int        INTERVAL = 5;  // second

    std::mutex mtx;
    std::condition_variable cv;
    bool g_keyIn = false;

    void exitApp()
    {
        std::unique_lock<std::mutex> lck(mtx);
        g_keyIn = true;
        cv.notify_one();
    }

    mxnet2::License mxnet2License(USERCODE, APPSLOT, INTERVAL, exitApp);

    void read_key()
    {
        char ch;
        std::cin.get(ch);

        exitApp();
    }

    void waitForKeyPress()
    {
        std::cout << "*** Enterで終了 *** " << std::endl;

        //Enterキーが押されるまで止める
        std::thread readKey(read_key);
        readKey.detach();

        std::unique_lock<std::mutex> lck(mtx);
        cv.wait(lck, [] { return g_keyIn; });
    }

    static int app_main()
    {
        int retVal = 0;


        try {

            //MxNet2License バックグラウンド更新開始（５秒毎に LogIn_MatrixNet 呼出してライセンス更新)
            mxnet2License.startUpdate();


            //フォアグラウンドでプログラム処理
            //Enterキーが押されるまで待つ（またはプログラム開始コードに置き換える）
            waitForKeyPress();
            std::cout <<  "終了します"  << std::endl;


            mxnet2License.stopUpdate();
            return 0;
        }
        catch (std::exception& e)
        {
            std::cout << "Exception raised: " << e.what() << std::endl;
            retVal = -1;
        }

        return retVal;
    }

}


int main()
{
     setlocale(LC_CTYPE, "ja_JP.UTF-8");

// mxnetapi.ini 設定の上書
/*
      if( sampleApp::mxnet2License.setIp("mxnet2a.local" ) == false )
      {
	std::cout << "Failed to set IP. IP will be read from mxnetapi.ini" << std::endl;
      }
      else
	    sampleApp::mxnet2License.setPort(12345);
*/

     if( false == sampleApp::mxnet2License.get() )
     {
        std::cout << "ライセンスを取得できませんでした" << std::endl;
        return -1;
     }

    //プログラム本体の処理開始
    return sampleApp::app_main();


}
